module Main where

import Corba
import IO
import IOExts
import Test

--------------------------------
--           TESTS
--------------------------------

assert_no_exception NoException = return ()
assert_no_exception ex = error ("ASSERT: Exception " ++ (show ex))

call_test_args obj
	= corba_invoke obj "test_args" [(InArg "arg1" (corba_String__encode "foo")), (OutArg "arg2" (corba_String__encode "")), (InOutArg "arg3" (corba_Long__encode 11))] corba_Long__tc []

call_echo obj 
	= corba_invoke obj "echo" [(InArg "what" (CorbaString "foo"))] (Tk_String 0) []

call_echo_tc obj 
	= corba_invoke obj "echo_tc" [(InArg "what" (CorbaTypeCode tc))] (Tk_TypeCode) []
	where
	tc = (Tk_Struct "IDL:Foo:1.0" "foo" [("aa", Tk_Long), ("bb", (Tk_String 0))])

call_deposit obj 
	= corba_invoke obj "deposit" [(InArg "amount" (CorbaLong 10))] Tk_Void []

call_self obj 
	= corba_invoke obj "self" [] (Tk_ObjRef "IDL:Foo:1.0" "xxx") []

call_balance obj 
	= corba_invoke obj "balance" [] Tk_Long []

call_balance2 obj 
	= corba_invoke obj "balance2" [(OutArgTemplate "balance" Tk_Long)] Tk_Void []

call_echo_objref obj
	= corba_invoke obj "echo_objref" [(InArg "what" (CorbaObjRef obj))] (Tk_ObjRef "IDL:Useless:1.0" "Useless") []

call_echo_struct obj = corba_invoke obj "echo_struct" [(InArg "what" (CorbaStruct tc members))] tc []
	where
		tc = (Tk_Struct "IDL:Foo:1.0" "foo" [
			("aaa", Tk_Short),
			("bbb", (Tk_Long)),
			("ccc", (Tk_String 0)), 
			("eee", (Tk_Sequence 0 (Tk_Long))),
			("ggg", corba_Object__tc)
			])
		members = [
			("aaa", (CorbaShort 5)),
			("bbb", (CorbaLong 2)),
			("ccc", (CorbaString "3")),
			("eee", (CorbaSequence (Tk_Sequence 0 (Tk_Long)) [(CorbaLong 1), (CorbaLong 2)])),
			("ggg", (CorbaObjRef obj))
			]

call_echo_seq obj = corba_invoke obj "echo_seq" [(InArg "what" (CorbaSequence tc members))] tc []
	where
	tc = (Tk_Sequence 0 (Tk_Long))
	members = [(CorbaLong 0), (CorbaLong 1), (CorbaLong 2)]

call_echo_union obj = corba_invoke obj "echo_union" [(InArg "what" (CorbaUnion tc (CorbaShort 0) (CorbaShort 2)))] tc []
	where
	tc = (Tk_Union "IDL:Foo:1.0" "bar" (Tk_Short) [("aaa", (CorbaShort 0), (Tk_Short)), ("bbb", (CorbaShort 1), (Tk_Long))])

call_echo_union_default obj = corba_invoke obj "echo_union" [(InArg "what" (CorbaUnion tc (CorbaShort 3) (CorbaChar 'a')))] tc []
	where
	tc = (Tk_Union "IDL:Foo:1.0" "bar" (Tk_Short) [("aaa", (CorbaShort 0), (Tk_Short)), ("bbb", (CorbaShort 1), (Tk_Long)), ("bbb", (CorbaShort 2), (Tk_Long)), ("ccc", (CorbaOctet 0), (Tk_Char))])

call_echo_enum obj = corba_invoke obj "echo_enum" [(InArg "what" (CorbaEnum tc "SECOND"))] tc []
	where
	tc = Tk_Enum "IDL:Foo:1.0" "foo" ["FIRST", "SECOND", "THIRD"]

call_echo_array obj = corba_invoke obj "echo_array" [(InArg "what" value)] tc []
	where
	tc = (Tk_Array 2 member_tc)
	member_tc = (Tk_Array 3 Tk_Long)
	value = (CorbaArray tc [
		(CorbaArray member_tc [(CorbaLong 1), (CorbaLong 2), (CorbaLong 3)]),
		(CorbaArray member_tc [(CorbaLong 4), (CorbaLong 5), (CorbaLong 6)])
				])

call_test_exit obj = corba_invoke obj "test_exit" [] tc []
	where
	tc = Tk_Void

call__get_an_attribute obj
	= corba_invoke obj "_get_an_attribute"[](Tk_String 0) []

call__set_an_attribute obj
	= corba_invoke obj "_set_an_attribute" [
		(InArg "attr" (corba_String__encode "foo"))
		] corba_Void__tc []


call_raise_exception obj
	= corba_invoke obj "raise_exception" [] (Tk_Void) [(Tk_Exception "IDL:Useless/Useless_Exception:1.0" "Useless_Exception" [("member1", Tk_Long), ("member2", Tk_Short)])]

--call_echo_recursive obj
--	= Useless_echo_recursive obj e1
--	where
--		e1 = {Useless_Recursive_struct | aaa = 1, bbb = {Useless_Recursive_struct_Nested | ccc = [e2, e3]}}
--		e2 = {Useless_Recursive_struct | aaa = 4, bbb = {Useless_Recursive_struct_Nested | ccc = []}}
--		e3 = {Useless_Recursive_struct | aaa = 3, bbb = {Useless_Recursive_struct_Nested | ccc = [e4]}}
--		e4 = {Useless_Recursive_struct | aaa = 2, bbb = {Useless_Recursive_struct_Nested | ccc = []}}

test_invoke2 obj
	= do
		print "balance()..."
		(res5, ex, out) <- call_balance obj
		print "balance2()..."
		(res7, ex, out) <- call_balance2 obj
		print "OK."

test_invoke obj
	= do
		print "test_args()..."
		(res1, ex, out) <- call_test_args obj
		assert_no_exception ex
		print "echo_objref()..."
		(res2, ex, out) <- call_echo_objref obj
		assert_no_exception ex
		print "echo_tc()..."
		(res3, ex, out) <- call_echo_tc obj
		assert_no_exception ex
		print "deposit()..."
		(res4, ex, out) <- call_deposit obj
		assert_no_exception ex
		print "balance()..."
		(res5, ex, out) <- call_balance obj
		assert_no_exception ex
		print "self()..."
		(res6, ex, out) <- call_self obj
		assert_no_exception ex
		print "balance2()..."
		(res7, ex, out) <- call_balance2 obj 	
		assert_no_exception ex
		print "echo_struct()..."
		(res8, ex, out) <- call_echo_struct obj
		print "echo_seq()..."
		(res9, ex, out) <- call_echo_seq obj	
		print "echo_union()..."
		(res10, ex, out) <- call_echo_union obj	
		print "echo_union_default()..."
		(res11, ex, out) <- call_echo_union_default obj	
		print "echo_enum()..."
		(res12, ex, out) <- call_echo_enum obj	
		print "set_an_attribute()..."
		(res13, ex, out) <- call__set_an_attribute obj	
		print "get_an_attribute()..."
		(res14, ex, out) <- call__get_an_attribute obj	
		print "raise_exception()..."
		(res15, ex, out) <- call_raise_exception obj	
		print "test_exit()..."
		(res15, ex, out) <- call_test_exit obj	
		print "OK."
		print "Results:"
		print res5

--------------------------------
--          MAIN
--------------------------------

m = do
		(orb,_)	<- corba_ORB_init []
--	 	(Just obj) <- corba_ORB_bind orb "IDL:Useless:1.0" "inet:localhost:4242"
		(Just obj) <- readIORFromFile orb "server.ior"
		test_invoke obj
--		call_echo obj

main = m

-- Local Variables:
-- tab-width: 4
-- End:




